package net.sf.sql2java.common.parameters.handlers;

import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.sf.sql2java.data.node.Nodelet;
import net.sf.sql2java.utils.ClassUtils;
import net.sf.sql2java.utils.StringUtils;

public class NodeParameterHandler implements ParameterHandler {
	private Map<String, Nodelet> nodes;

// -- Constructors ------------------------------------------------------------
	
	public NodeParameterHandler(Nodelet node) {
		this.nodes = new HashMap<String, Nodelet>();
		
		addNode(node);
	}
	
// -- Parameter Event Handling ------------------------------------------------
	
	/**
	 * Handle a parameter occurance.
	 * 
	 * A parameter found by this handler will be a parameter of the form
	 * <b>name</b> or <b>schema.name</b>. The parameter will be separated in
	 * different parts, using the dot (.) as a delimiter. Every parameter part
	 * will be a property of the previous one, beginning with the processable
	 * itself.
	 * <p>
	 * Maybe a few examples to make it less complex:
	 * <ul>
	 * <li><b>name</b> means the property 'name' of the current processable.
	 * <li><b>schema.name</b> means the property 'name' of the property 'schema' 
	 * 						  of the current processable.
	 * </ul>
	 */
	@Override
	public String onParameter(String content, String parameter) {
		// -- create the result
		String result = content;
		String replacement;
		String key = parameter;
		
		if (key.indexOf("@") != -1)
			key = key.substring(0, key.indexOf("@"));
		
		try {
			// -- get the node key
			String nodeKey = key.split("#")[0];
			
			if (! nodes.containsKey(nodeKey))
				throw new Exception(
						String.format("No node found of type %s while injecting parameter", nodeKey));
			
			// -- get the node
			Nodelet node = nodes.get(nodeKey);

			// -- get the parameter value
			Object parameterValue = ClassUtils.getValue(key.substring(key.indexOf('#') + 1), node);
			
			if (parameterValue == null) replacement = "null";
			else replacement = parameterValue.toString();
			
			if (parameter.indexOf("@") != -1) {
				// -- Lets modify the result
		        Pattern p = Pattern.compile("(\\@.{2})", Pattern.CASE_INSENSITIVE);
		        Matcher m = p.matcher(parameter);
		        
		        while (m.find()) {
		            String modifier = m.group();
		            
		            if (modifier.equalsIgnoreCase("@lc"))
		                replacement = replacement.toLowerCase();
		            else if (modifier.equalsIgnoreCase("@uc"))
		                replacement = replacement.toUpperCase();
		            else if (modifier.equalsIgnoreCase("@ca"))
		                replacement = StringUtils.capitalize(replacement);
		            else if (modifier.equalsIgnoreCase("@dc"))
		                replacement = StringUtils.decapitalize(replacement);
		        }
			}
			
			// -- replace all parameter occurances
			result = result.replaceAll(
					StringUtils.parameterize(parameter), 
					replacement
			);
			
			// -- return the result
			return result;
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

// -- Setters -----------------------------------------------------------------
	
	public void addNode(Nodelet node) {
		nodes.put(node.getType(), node);
		
		Nodelet currentNode = node;
		while (currentNode.getParent() != null) {
			// -- add the parent
			nodes.put(currentNode.getParent().getType(), currentNode.getParent());
			
			currentNode = currentNode.getParent();
		}
	}
}
