package net.sf.sql2java.common.store;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import net.sf.sql2java.common.beans.Processable;

public class BasicProcessorStore implements ProcessorStore {
	public static final Processable ROOT = new Processable("root", "root", null, "root");
	
	protected Map<String, Map<String, List<Processable>>> stringedMap;
	protected Map<String, Processable> keyedMap;
	
// -- Constructors ------------------------------------------------------------
	
	public BasicProcessorStore() {
		stringedMap = new HashMap<String, Map<String, List<Processable>>>();
		keyedMap = new HashMap<String, Processable>();
	}
	
// -- Processables by String ---------------------------------------------------
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public List<Processable> getProcessables(String scope) {
		List<Processable> result = new LinkedList<Processable>();
		
		Map<String, List<Processable>> processables = stringedMap.get(scope);
		
		if (processables != null)
			for (List<Processable> children : processables.values())
				result.addAll(children);
		
		return result;
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public List<Processable> getProcessables(String scope, String parentKey) {
		return stringedMap.get(scope).get(parentKey);
	}

// -- Processable Manipulation ------------------------------------------------
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public void addProcessable(Processable processable) throws StoreException {
		// -- check if the processable has a parent. If not, add it to the root
		if (processable.getParent() == null)
			processable.setParent(ROOT);
		
		// -- throw an exception if the processable's key already exists
		if (keyedMap.containsKey(processable.getKey()))
			throw new StoreException("Duplicate Key when adding parameter!");
		
		// -- attach the processable to it's parent
		processable.getParent().addReference(processable);
		
		// -- add the processable to the keyedMap
		keyedMap.put(processable.getKey(), processable);
		
		// -- add the processable to the StringdMap
		if (! stringedMap.containsKey(processable.getScope()))
			stringedMap.put(processable.getScope(), new HashMap<String, List<Processable>>());
		
		if (! stringedMap.get(processable.getScope()).containsKey(processable.getParent().getKey()))
			stringedMap.get(processable.getScope()).put(processable.getParent().getKey(), new LinkedList<Processable>());
		
		stringedMap.get(processable.getScope()).get(processable.getParent().getKey()).add(processable);
	}
		
	/**
	 * {@inheritDoc}
	 */
	@Override
	public void removeProcessable(Processable processable) {
		// -- remove the processable from it's parent
		processable.getParent().removeReference(processable);
		
		// -- remove the processable from the keyedMap
		keyedMap.remove(processable.getKey());
		
		// -- remove the processable from the StringdMap
		if (stringedMap.containsKey(processable.getScope())) {
			// -- remove the processable
			stringedMap.get(processable.getScope()).remove(processable);
		
			// -- clear the map of empty lists
			if (stringedMap.get(processable.getScope()).size() == 0)
				stringedMap.remove(processable.getScope());
		}
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public Processable duplicateProcessable(Processable processable) {
		return processable.clone();
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public Processable getProcessable(String key) {
		return keyedMap.get(key);
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean hasProcessable(String key) {
		return keyedMap.containsKey(key);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void save(Processable processable) throws StoreException {
		if (hasProcessable(processable.getKey()))
			removeProcessable(processable);
		
		addProcessable(processable);
	}

	
	
}
